from datetime import datetime, timedelta, timezone
from ..database import db
from secrets import compare_digest
from flaschengeist import logger


class AccessToken(db.Model):
    """ Model for an AccessToken

        Args:
            expires: Is a Datetime from current Time.
            user:   Is an User.
            token:  String to verify access later.
    """
    __tablename__ = 'session'
    id = db.Column(db.Integer, primary_key=True)
    user_id = db.Column(db.Integer, db.ForeignKey('user.id'))
    user = db.relationship("User", back_populates="sessions")

    expires = db.Column(db.DateTime)
    token = db.Column(db.String(32), unique=True)
    lifetime = db.Column(db.Integer)
    browser = db.Column(db.String(30))
    platform = db.Column(db.String(30))

    def refresh(self):
        """ Update the Timestamp

            Update the Timestamp to the current Time.
        """
        logger.debug("update timestamp from access token {{ {} }}".format(self))
        self.expires = datetime.utcnow() + timedelta(seconds=self.lifetime)

    def serialize(self):
        """ Create Dic to dump in JSON

            Returns:
                A Dic with static Attributes.
        """
        return {
            "token": self.token,
            "expires": self.expires.replace(tzinfo=timezone.utc),
            "lifetime": self.lifetime,
            "browser": self.browser,
            "platform": self.platform
        }

    def __eq__(self, token):
        return compare_digest(self.token, token)

    def __str__(self):
        return "AccessToken(user={}, token={}, expires={}, lifetime={})".format(
            self.user, self.token, self.expires, self.lifetime)
